import numpy as np


def i_trick(fun, x, param):
    """Calculates the Jacobian of function "fun" in "x" using the imaginary trick

    returns:    
    F: the function value in "x"
    J: Jacobian of "fun" in "x"
    """

    # Definining the step length
    h = 1e-100

    # Evaluate once to get dimension
    F = fun(x, param)

    nz = np.size(x)

    # Initialize Jacobian
    J = np.zeros((np.size(F),nz))

    # In each loop we obtain one column of the Jacobian (1j*h is the seed)
    for k in range(nz):
        x_temp = x.astype(complex)
        x_temp[k] += 1j*h
        J[:,k] = np.imag(fun(x_temp, param)) / h

    return (F, J)
